/******************************************************************************
 *                      Sources BCB - www.developpez.com                      *
 * -------------------------------------------------------------------------- *
 *                                                                            *
 * LZWCDFunctions.h                                                           *
 * Version: 1.0.0 mise  jour le 13 fvrier 2005                              *
 * Copyright Cryptonyx (http://five.chez.tiscali.fr)                          *
 *                                                                            * 
 * Test avec:                                                                *
 * SE: WinXP                                                                  *
 * IDE: BCB5                                                                  * 
 *                                                                            *
 * -------------------------------------------------------------------------- *
 *                                                                            * 
 * Cet utilitaire illustre une compression/dcompression avec l'algorithme    *
 * de Liv, Zempel et Welch (LZW)                                              *
 *                                                                            *
 * -------------------------------------------------------------------------- * 
 *                                                                            *
 * Retrouvez d'autres ressources BCB sur :                                    *
 *  - http://c.developpez.com/sources/bcb/                                    *
 *  - http://c.developpez.com/faq/bcb/                                        * 
 * Ainsi que sur les forums (http://www.developpez.net/forums/)               *
 *                                                                            *
 ******************************************************************************/

#ifndef LZWCDFunctionsH
#define LZWCDFunctionsH

#include <dstring.h>
//--------------------------------------------------------------------------
// Constantes
#define PATH_DELIMITER      "\\"
#define MAXIMUM_ELEMENTS    256
#define VERSION 0x10
//---------------------------------------------------------------------------
// Codes d'erreurs
typedef enum { eCorrect, eOpenReadFailed, eOpenWriteFailed, eEmptyFile, eWriteFailed,
	eCreateFailed, eReadFailed, eBadCode, eBadList, eBadVersion, eBadHeader, eBadFileName } LZWError;

//--------------------------------------------------------------------------
// Dfinition des structures de donnes
//--------------------------------------------------------------------------
// Structure ncessaires  la lecture et  l'criture des enttes
union long_or_char {
    unsigned long int LongSize;
    char CharSize[4];
};
typedef union long_or_char LongOrChar;
union short_or_char {
    unsigned short int ShortValue;
    char CharSize[2];
};
typedef union short_or_char ShortOrChar;

#define BITS 12                         // Choix du nombre de bits pour la table.
#define HASHING_SHIFT (BITS - 8)        // Cette valeur affecte plusieurs constantes.
#define MAX_VALUE ((1 << BITS) - 1)
#define MAX_CODE (MAX_VALUE - 1)

#if BITS == 14
  #define TABLE_SIZE 18041      // La taille de la table des chanes doit tre
#endif                          // un nombre premier suprieur  2 ** BITS
#if BITS == 13
  #define TABLE_SIZE 9029
#endif
#if BITS <= 12
  #define TABLE_SIZE 5021
#endif

//--------------------------------------------------------------------------
// Classe TLZWClass permettant de raliser la (d)compression selon
// l'algorithme de Lempel, Ziv et Welch (LZW)
//--------------------------------------------------------------------------
class TLZWClass
{
private :
    AnsiString Source;                      // Fichier source
    AnsiString Destination;                 // Fichier de destination
    int SourceFileHandle;                   // Adresse du descripteur du fichier source
    int DestinationFileHandle;              // Adresse du descripteur du fichier de destination
    unsigned char *InputFileBuffer;         // Tampon pour le fichier d'entre
    unsigned char *OutputFileBuffer;        // Tampon pour le fichier de sortie
    unsigned short int NumberOfElements;    // Nombre d'lments diffrents du fichier
    unsigned long int ExpandedSize;         // Taille du fichier original
    unsigned long int CompressedSize;       // Taille du fichier compress avec entte
    unsigned short int EndOfHeader;         // Longueur de l'entte
    unsigned int ErrorCode;                 // Code de la dernire erreur survenue
    AnsiString ErrorString;                 // Message associ  la dernire erreur survenue
    unsigned short int CodeValueTable[TABLE_SIZE + 1];          // Table des codes
    unsigned short int PrefixCodeTable[TABLE_SIZE + 1];         // Table des prefixes
    unsigned char AppendedCharacterTable[TABLE_SIZE + 1];       // Table des caractres d'extension
    unsigned char StringStack[TABLE_SIZE + 1];                  // Table des chanes de caractres dcodes

    bool HeaderGenerate();                      // Ecrit l'entte dans le fichier de destination
    bool HeaderRead();                          // Lit l'entte  partir du fichier source
    unsigned short int InputCode(unsigned long int *CodePosition);
    unsigned long int OutputCode(unsigned long int CodePosition, unsigned short int Code);
    unsigned short int FindMatch(unsigned short int HashPrefix, unsigned short int HashCharacter);
    int  DecodeString(int StackDepth, unsigned int Code);
    bool Initialize(const AnsiString InputFile, const AnsiString OutputFile);
    void Reset();

public :
    bool FileCompress(const AnsiString InputFile, const AnsiString OutputFile);      // Compresse un fichier
    bool FileDecompress(const AnsiString InputFile, const AnsiString OutputFile);    // Dcompresse un fichier
    AnsiString GetLastError();
    __fastcall TLZWClass();
};
#endif
